from forge import protos
from forge.config import config

stub = protos.WalletRpcStub(config.get_grpc_channel())

__wallet_type = protos.WalletType(pk=0, hash=1, address=1)


def create_wallet(
        wallet_type=__wallet_type, moniker='',
        passphrase='', req=None,
):
    """GRPC call to create a wallet on Forge

    Note:
        the wallet created by this GRPC call will save encrypted private key
        on Forge node.

    Args:
        wallet_type(:obj:`WalletType`): the wallet type to create
        moniker(string): optional, nickname for wallet. Wallet will not be
            declared if no moniker is provided.
        passphrase(string): required, password to encrypt wallet
        req(:obj:`RequestCreateWallet`): completed request

    Returns:
        ResponseCreateWallet

    """
    if req is not None:
        return stub.create_wallet(req)
    else:
        req_kwargs = {
            'type': wallet_type,
            'moniker': moniker,
            'passphrase': passphrase,
        }
        return stub.create_wallet(
            protos.RequestCreateWallet(**req_kwargs),
        )


def load_wallet(address, passphrase, req=None):
    """GRPC call to load wallet stored on Forge

    Args:
        address(string): address of the wallet stored
        passphrase(string): password used to create the wallet
        req(:obj:`ReqeustLoadWallet`): completed request

    Returns:
        ResponseLoadWallet

    """
    if req is not None:
        return stub.load_wallet(req)
    else:
        req_kwargs = {
            'address': address,
            'passphrase': passphrase,
        }
        return stub.load_wallet(
            protos.RequestLoadWallet(**req_kwargs),
        )


def recover_wallet(
        passphrase, moniker, data,
        wallet_type=__wallet_type, req=None,
):
    """GRPC call to recover a wallet on forge

    Note:
        This GRPC call requires your private key. Please be careful when
        providing that information

    Args:
        passphrase(string): new passphrase to encrypt this wallet on Forge
        moniker(string): nickname for this wallet
        data(bytes): seed word or private key
        wallet_type(:obj:`WalletType`): deprecated
        req(:obj:`ReqeustRecoverWallet`): completed request

    Returns:
        ResponseRecoverWallet

    """

    if req is not None:
        return stub.recover_wallet(req)
    else:
        req_kwargs = {
            'data': data,
            'type': wallet_type,
            'passphrase': passphrase,
            'moniker': moniker,
        }
        req = protos.RequestRecoverWallet(**req_kwargs)
        print(req)
        return stub.recover_wallet(req)


def list_wallet():
    """ GRPC call to list al wallets on current Forge Node

    Returns
        ResponseListWallets(stream)

    """
    return stub.list_wallet(protos.RequestListWallet())


def remove_wallet(address, req=None):
    """GRPC call to remove wallet with given address

    Args:
        address(string): address of wallet to be removed from Forge
        req(:obj:`RequestRemoveWallet`): completed request

    Returns:
        ResponseRemoveWallet

    """
    if req is not None:
        return stub.remove_wallet(req)
    else:
        req_kwargs = {
            'address': address,
        }
        return stub.remove_wallet(
            protos.RequestRemoveWallet(**req_kwargs),
        )


def declare_node(req):
    """GRPC call to declare current node

    Args:
        req(:obj:`RequestDeclareNode`: completed request

    Returns:
        ResponseDeclareNode

    """
    if req:
        return stub.declare_node(req)
    else:
        return stub.declare_node(protos.RequestDeclareNode())



from forge import protos
from forge.config import config
from forge.utils import utils

stub = protos.ChainRpcStub(config.get_grpc_channel())


def create_tx(itx, from_address,
              wallet=None, token=None, req=None, nonce=1,
              ):
    """ GRPC call to build a complete transaction, including sender's pk and
    sender's signature

    Note:
        To sign a transaction successfully, either a wallet with private key or
        a token should be provided. However, this practice is not recommended
        for safety concern. Users should keep their own private keys and sign
        transactions locally.

    Args:
        itx (:obj:`google.protobuf.Any`): Inner transaction that should be
            included in this transaction
        from_address (string) : address of user responsible for sending this
            transactions
        wallet (:obj:`WalletInfo`) : user wallet
        token (string): token provided by forge for using wallets stored on
            forge
        req (:obj:`RequestCreateTx`): complete request
        nonce (int): optional, number of tx this account has sent

    Returns:
        ResponseCreateTx

    """

    if req is not None:
        return stub.create_tx(req)
    else:
        req_kwargs = {
            'itx': itx,
            'from': from_address,
            'nonce': nonce,
            'wallet': wallet,
            'token': token,
        }
        return stub.create_tx(protos.RequestCreateTx(**req_kwargs))


def send_tx(
        tx, wallet=None, token=None, commit=False, req=None,
):
    """GRPC call to send the included transaction

    Args:
        tx(:obj:`Transaction`): transaction to be sent
        wallet (:obj:`WalletInfo`) : user wallet
        token (string): token provided by forge for using wallets stored on
            forge
        commit(bool): option to wait until transaction was included in a block
        req(:obj: `RequestSendTx`): complete request

    Returns:
            ResponseSendTx

    """
    if req is not None:
        return stub.send_tx(req)
    else:
        req_kwargs = {
            'tx': tx,
            'wallet': wallet,
            'token': token,
            'commit': commit,
        }
        req = protos.RequestSendTx(**req_kwargs)
        return stub.send_tx(req)


def get_tx(tx_hash, reqs=None):
    """ GRPC call to get detailed information of a transaction

    Args:
        tx_hash(string or list[string]): hash of the transaction
        reqs (RequestGetTx): list of request

    Returns:
         ResponseGetTx(stream)

    """

    def to_req(item):
        if isinstance(item, protos.RequestGetTx):
            return item
        else:
            return protos.RequestGetTx(hash=item)

    if reqs is not None:
        return stub.get_tx(utils.to_iter(to_req, reqs))
    else:
        return stub.get_tx(utils.to_iter(to_req, tx_hash))


def get_block(height, req=None):
    """ GRPC call to get blocks information

    Args:
        height(int or list[int]): height of the block
        req(:obj: `RequestGetBlock`): sream of requests

    Returns:
        ResponseGetBlock(stream)

    """

    def to_req(item):
        if isinstance(item, protos.RequestGetBlock):
            return item
        else:
            return protos.RequestGetBlock(height=item)

    if req is not None:
        return stub.get_block(utils.to_iter(to_req, req))
    else:
        return stub.get_block(utils.to_iter(to_req, height))


def search(key, value, req=None):
    """GRPC call to search for specific key-value pair

    Args:
        key(string): key
        value(string): value
        req(:obj:`RequestSearch`): request

    Returns:
        ResponseSearch(stream)

    """
    if req is not None:
        return stub.search(req)
    else:
        req_kwargs = {
            'key': key,
            'value': value,
        }
        return stub.search(protos.RequestSearch(**req_kwargs))


def get_unconfirmed_tx(limit=1, req=None):
    """GRPC call to get currently unconfirmed transactions

    Args:
        limit(int): maximum number of transactions to get
        req(:obj:`RequestGetUnconfirmedTx`): request

    Returns:
        ResponseGetUnconfirmedTxs

    """
    if req is not None:
        return stub.get_unconfirmed_txs(req)
    else:
        return stub.get_unconfirmed_txs(
            protos.RequestGetUnconfirmedTxs(limit=limit),
        )


def get_chain_info(req=None):
    """ RPC call to get information about current chain

    Args:
        req(:obj:`RequestGetChainInfo`): request

    Returns:
        ResponseGetChainInfo

    """
    if req is not None:
        return stub.get_chain_info(req)
    else:
        return stub.get_chain_info(protos.RequestGetChainInfo())


def get_net_info(req=None):
    """RPC call to get information of the net

    Args:
        req(:obj:`RequestGetNetInfo`): Request

    Returns:
        ResponseGetNetInfo
    """
    if req is not None:
        return stub.get_net_info(req)
    else:
        return stub.get_net_info(protos.RequestGetNetInfo())


def get_validators_info(req=None):
    """GRPC call to get informations about al current validators

    Args:
        req(:obj:`RequestGetValidatorsInfo`: completed request

    Returns:
        ResponseGetValidatorsInfo

    """

    if req is not None:
        return stub.get_validators_info(req)
    else:
        return stub.get_validators_info(
            protos.RequestGetValidatorsInfo(),
        )


def get_config(req=None):
    """ RPC call to get detailed configuration current chain is using

    Args:
        req(:obj:`RequestGetConfig`): completed Request

    Returns:
        ResponseGetConfig

    """

    if req is not None:
        return stub.get_config(req)
    else:
        return stub.get_config(
            protos.RequestGetConfig(),
        )


def multisig(tx, wallet=None, token=None, data=None, req=None):
    """GRPC call to get multi-signature of a transaction. When executing this
    transactions, Forge will insert the address to `signatures` field as
    `Multisig.signer`, then create a signature of the entire transaction

    Args:
        tx(:obj:`Trasnaction`): transaction to be signed
        wallet (:obj:`WalletInfo`) : user wallet
        token (string): token provided by forge for using wallets stored on
            forge
        data(:obj:`google.protobuf.Any`): extra data to be included in multisig
        req(:obj:`RequestMultisig`): completed request

    Returns:
        ResponseMultisig

    """
    if req is not None:
        return stub.multisig(req)
    else:
        req = protos.RequestMultisig(
            tx=tx, wallet=wallet, token=token,
            data=data,
        )
        return stub.multisig(req)


def get_asset_address(
        sender_address, itx,
        wallet_type=None, req=None,
):
    """RPC call to get asset address calculated from createAssetTx and
    sender address

    Args:
        sender_address(string): address of the creator of this asset
        itx(:obj:`CreateAssetTx`): the inner transaction to create asset
        wallet_type(:obj:`WalletType`): deprecated
        req(:obj:`RequestGetAssetAddress`): completed request

    Returns:
        ResponseGetAssetAddress

    """
    if req is not None:
        return stub.get_asset_address(req)
    else:
        return stub.get_asset_address(
            protos.RequestGetAssetAddress(
                sender_address=sender_address, itx=itx,
                wallet_type=wallet_type,
            ),
        )


def get_blocks(min_height,
               max_height,
               empty_excluded=False,
               paging=None,
               req=None):
    """GRPC call to get information of blocks

    Args:
        min_height(int): minimum height of blocks
        max_height(int): maximum height of blocks
        empty_excluded(bool): whether to include empty blocks or not
        paging(:obj:`PageInput`): optional, paging preference
        req(:obj:`RequestGetBlocks`): completed request

    Returns:
        ResponseGetBlocks

    """
    if req:
        return stub.get_blocks(req)
    else:
        return stub.get_blocks(
            protos.RequestGetBlocks(
                paging=paging,
                min_height=min_height,
                max_height=max_height,
                empty_excluded=empty_excluded
            )
        )


def get_node_info(req=None):
    """GRPC call to get information of current node

    Args:
        req(:obj:`RequestGetNodeInfo`): completed request

    Returns:
        ResponseGetNodeInfo

    """
    if req:
        return stub.get_node_info(req)
    else:
        return stub.get_node_info(protos.RequestGetNodeInfo())


def sign_data(data, wallet, token, req=None):
    """ GRPC call to get signature for specific data

    Args:
        data(bytes): data needs to be signed
        wallet (:obj:`WalletInfo`) : user wallet
        token (string): token provided by forge for using wallets stored on
            forge
        req(:obj:`RequestSignData`): completed request

    Returns:
        ResponseSignData

    """
    if req:
        return stub.sign_data(req)
    else:
        return stub.sign_data(protos.RequestSignData(
            data=data, wallet=wallet, token=token
        ))



from forge import protos
from forge.config import config
from forge.utils import utils

stub = protos.FileRpcStub(config.get_grpc_channel())


def store_file(chunk, req=None):
    """GRPC call to store file

    Args:
        chunk(bytes or list[bytes]): file bytes to store
        req(:obj:`RequestStoreFile`): stream of completed request

    Returns:
        ResponseStoreFile

    """

    def to_req(item):
        if isinstance(item, protos.RequestStoreFile):
            return item
        else:
            return protos.RequestStoreFile(chunk=item)

    if req is not None:
        return stub.store_file(utils.to_iter(to_req, req))
    else:
        chunks = utils.to_iter(to_req, chunk)
        return stub.store_file(chunks)


def load_file(file_hash, req=None):
    """GRPC call to load stored file

    Args:
        file_hash(string): hash of stored file
        req(:obj:`RequestLoadFile`): completed request

    Returns:
        ResponseLoadFile(stream)

    """

    if req is not None:
        return stub.load_file(req)
    else:
        req_kwargs = {
            'hash': file_hash,
        }
        return stub.load_file(protos.RequestLoadFile(**req_kwargs))


def pin_file(file_hash, req=None):
    """GRPC call to pin file so Forge will keep the file

    Args:
        file_hash(string): hash of the file to pin
        req(:obj:`ReqeustPinFile`): completedRequest

    Returns:
        ResponsePinFile

    """
    if req:
        return stub.pin_file(req)
    else:
        return stub.pin_file(protos.RequestPinFile(hash=file_hash))



from forge import protos
from forge.config import config
from forge.utils import utils

stub = protos.StateRpcStub(config.get_grpc_channel())


def get_account_state(queries, reqs=None):
    """GRPC call to get detailed of account

    Args:
        queries(dict): dictionaries of requested parameters
        reqs(list[:obj:`RequestGetAccountState`): stream of completed request

    Returns:
        ResponseGetAccountState(stream)

    """

    def to_req(item):
        if isinstance(item, protos.RequestGetAccountState):
            return item
        else:
            kwargs = {
                'address': item.get('address'),
                'keys': item.get('keys', []),
            }
            return protos.RequestGetAccountState(**kwargs)

    requests = reqs if reqs else utils.to_iter(to_req, queries)

    return stub.get_account_state(requests)


def get_asset_state(queries, reqs=None):
    """GRPC call to get detailed of asset

    Args:
        queries(dict): dictionaries of requested parameters
        reqs(list[:obj:`RequestGetAssetState`): stream of completed request

    Returns:
        ResponseGetAssetState(stream)
    """

    def to_req(item):
        if isinstance(item, protos.RequestGetAssetState):
            return item
        else:
            kwargs = {
                'address': item.get('address'),
                'keys': item.get('keys', []),
                # 'app_hash': item.get('app_hash', ''),
            }
            req = protos.RequestGetAssetState(**kwargs)
            return req

    requests = reqs if reqs else utils.to_iter(to_req, queries)

    return stub.get_asset_state(requests)


def get_stake_state(queries, reqs=None):
    """GRPC call to get detailed of stake

    Args:
        queries(dict): dictionaries of requested parameters
        reqs(list[:obj:`RequestGetStakeState`): stream of completed request

    Returns:
        ResponseGetStakeState(stream)

    """

    def to_req(item):
        if isinstance(item, protos.RequestGetStakeState):
            return item
        else:
            kwargs = {
                'address': item.get('address'),
                'keys': item.get('keys', []),
            }
            return protos.RequestGetStakeState(**kwargs)

    requests = reqs if reqs else utils.to_iter(to_req, queries)
    return stub.get_stake_state(requests)


def get_forge_state(keys=[], height=None, req=None):
    """ GRPC call to get forge state

    Args:
        keys(list[string]): optional, list of keys to receive. GRPC returns
            all keys if not specified.
        height(int): optional, forge state of specific block height
        req(:obj:`RequestGetForgeState`): completed request

    Returns:
        ResponseGetForgeState

    """
    if req is not None:
        return stub.get_forge_state(req)
    else:
        req_kwargs = {
            'keys': keys,
            'height': height,
        }
        return stub.get_forge_state(
            protos.RequestGetForgeState(**req_kwargs),
        )



import logging
from inspect import getmembers
from inspect import isfunction

from forge import protos
from forge import utils
from forge.rpc import chain
from forge.rpc import event
from forge.rpc import file
from forge.rpc import state
from forge.rpc import statistic
from forge.rpc import wallet

logger = logging.getLogger('forge-rpc')


def __get_module_functions(module):
    return {item[0]: getattr(module, item[0]) for item in
            getmembers(module, isfunction)}


__all_services = {**__get_module_functions(chain),
                  **__get_module_functions(state),
                  **__get_module_functions(event),
                  **__get_module_functions(file),
                  **__get_module_functions(wallet),
                  **__get_module_functions(statistic)}

for name, func in __all_services.items():
    vars()[name] = func


__wallet_type = protos.WalletType(pk=0, hash=1, address=1)


def send_itx(type_url, itx, wallet, token, nonce=1):
    """GRPC call to send inner transaction

    Args:
        type_url(string): type_url for this itx
        itx(:obj:`protos.object`): inner transactions defined in protos
        wallet(:obj:`WalletInfo`): sender's wallet
        token(string): sender's token
        nonce(int): need to be set to 0 if itx is pokeTx

    Returns:
        ResponseSendTx

    """
    encoded_itx = utils.encode_to_any(type_url, itx)
    tx = create_tx(
        itx=encoded_itx, from_address=wallet.address,
        wallet=wallet, token=token, nonce=nonce
    )
    return send_tx(tx.tx)


def create_asset(type_url, asset, wallet, token):
    """GRPC call to create asset

    Args:
        type_url(string): type_url for this itx
        asset(object): asset to be included in itx, can be string, bytes,
            or protobuf objects
        wallet(:obj:`WalletInfo`): sender's wallet
        token(string): sender's token

    Returns:
        ResponseSendTx

    """
    encoded_asset = utils.encode_to_any(type_url, asset)
    create_asset_itx = utils.encode_to_any(
        type_url='fg:t:create_asset',
        data=protos.CreateAssetTx(data=encoded_asset),
    )
    tx = create_tx(
        create_asset_itx, wallet.address, wallet, token,
    )
    return send_tx(tx.tx)


def update_asset(type_url, address, asset, wallet, token):
    """GRPC call to create asset

        Args:
            type_url(string): type_url for this itx
            address(string): address of asset to update
            asset(object): asset to be updated in itx, can be string, bytes,
                or protobuf objects
            wallet(:obj:`WalletInfo`): sender's wallet
            token(string): sender's token

        Returns:
            ResponseSendTx

        """
    encoded_asset = utils.encode_to_any(type_url, asset)
    update_asset_itx = utils.encode_to_any(
        type_url='fg:t:update_asset',
        data=protos.UpdateAssetTx(
            address=address,
            data=encoded_asset,
        ),
    )
    tx = create_tx(
        update_asset_itx, wallet.address, wallet, token,
    )
    return send_tx(tx.tx)


def get_single_account_state(address):
    """GRPC call to get account state of a single address

    Args:
        address(string): address of the account

    Returns:
        AccountState

    """
    if address:
        accounts = get_account_state({'address': address})
        account = next(accounts)
        if utils.is_proto_empty(account):
            return None
        else:
            return account.state


def get_single_tx_info(hash):
    """GRPC call to get transaction state of a single hash

    Args:
        hash(string): hash of the transaction

    Returns:
        TransactionInfo

    """
    if hash:
        infos = get_tx(hash)
        info = next(infos)
        if utils.is_proto_empty(info):
            return None
        else:
            return info.info


def get_single_asset_state(address):
    """GRPC call to get asset state of a single address

    Args:
        address(string): address of the asset

    Returns:
        AssetState

    """
    if address:
        assets = get_asset_state({'address': address})
        asset = next(assets)
        if not utils.is_proto_empty(asset):
            return asset.state



from forge import protos
from forge.config import config

stub = protos.StatisticRpcStub(config.get_grpc_channel())


def get_forge_statistics(day_info=None, date=None, req=None):
    """GRPC call to get statistics about forge

    Args:
        day_info(:obj:`Byday`): optional, date filter for information
        date(:obj:`ByHour`): optional, information returned by hour in specific
         day
        req(:obj:`RequestGetForgeStatics`): completed request

    Returns:
        ResponseForgeStatistics

    """
    if req:
        return stub.get_forge_statistics(req)
    elif day_info:
        return stub.get_forge_statistics(
            protos.RequestGetForgeStatistics(day_info=day_info))
    elif date:
        return stub.get_forge_statistics(
            protos.RequestGetForgeStatistics(date=date))
    else:
        raise ValueError('Please pass either day_info or date for grpc call'
                         'get_forge_statistics.')


def list_assets(owner_address, paging=None, req=None):
    """GRPC call to list all assets under the given account address

    Args:
        owner_address(string): target account address
        paging(:obj:`PageInput`): optional, paging preference
        req(:obj:`RequestListAssets`): completed request

    Returns:
        ResponseListAssets

    """
    if req:
        return stub.list_assets(req)
    else:
        return stub.list_assets(
            protos.RequestListAssets(owner_address=owner_address,
                                     paging=paging))


def get_stakes(address_filter, paging=None, req=None):
    """ GRPC call to get stakes

    Args:
        address_filter(string): filter stakes to get
        paging(:obj:`PageInput`): paging preferences
        req(:obj:`RequestGetStakes`): completed request

    Returns:
        ResponseGetStakes

    """
    if req:
        return stub.get_stakes(req)
    else:
        return stub.get_stakes(
            protos.RequestGetStakes(address_filter=address_filter,
                                    paging=paging))


def get_top_accounts(paging=None, req=None):
    """GRPC call to get top accounts

    Args:
        paging(:obj:`PageInput`): paging preferences
        req(:obj:`RequestGetTopAccounts`): completed request

    Returns:
        ResponseGetTopAccounts

    """
    if req:
        return stub.get_top_accounts(req)
    else:
        return stub.get_top_accounts(
            protos.RequestGetTopAccounts(paging=paging))


def list_blocks(paging=None, proposer=None, time_filter=None,
                height_filter=None, num_txs_filter=None,
                num_invalid_txs_filter=None, req=None):
    """GRPC call to list information of blocks

    Args:
        paging(:obj:`PageInput`): paging preferences
        proposer(string): address that proposed the block
        time_filter(:obj:`TimeFilter`): time filter
        height_filter(:obj: `HeightFilter`): height filter
        num_txs_filter(:obj:`NumTxsFilter`): number of transaction filter
        num_invalid_txs_filter(:obj:`NumInvalidTxsFilter` ): number of invalie
            transaction filter
        req(:obj:`RequestListBlocks`): completed request

    Returns:
        ResponseListBlocks

    """
    if req:
        return stub.list_blocks(req)
    else:
        return stub.list_blocks(protos.RequestListBlocks(
            paging=paging, proposer=proposer, time_filter=time_filter,
            height_filter=height_filter, num_txs_filter=num_txs_filter,
            num_invalid_txs_filter=num_invalid_txs_filter
        ))


def get_health_status():
    """GRPC call to get Forge health status

    Returns:
        ResponseGetHealthStatus

    """
    return stub.get_health_status(protos.RequestGetHealthStatus())


def list_asset_transactions(address, paging=None, req=None):
    """GRPC call to list transactions related to specific asset

    Args:
        address(string): asset address
        paging(:obj:`PageInput`): paging preferences
        req(:obj:`RequestListAssetTransactions`): completed request

    Returns:
        ResponseListAssetTransaction

    """
    if req:
        return stub.list_asset_transactions(req)
    else:
        return stub.list_asset_transactions(
            protos.RequestListAssetTransactions(
                address=address,
                paging=paging,
            ),
        )


def list_transactions(
        address_filter=None, time_filter=None,
        type_filter=None, paging=None,
):
    """GRPC call to list transactions

    Args:
        address_filter(string): address filter
        time_filter(:obj:`TimeFilter`): time filter
        type_filter(:obj:`TypeFilter`): type filter
        paging(:obj:`PagingInput`): paging preference

    Returns:
        ResponseListTransactions

    """
    req = protos.RequestListTransactions(
        paging=paging,
        time_filter=time_filter,
        address_filter=address_filter,
        type_filter=type_filter,
    )
    return stub.list_transactions(req)



import logging
import sqlite3
from sqlite3 import Error

from event_chain.config import config

logger = logging.getLogger('db-helper')


def create_connection():
    """ create a database connection to the SQLite database
        specified by the db_file
    :param db_file: database file
    :return: Connection object or None
    """
    try:
        conn = sqlite3.connect(config.db_path)
        return conn
    except Error as e:
        logger.error(e)
    return None


def init_db(conn):
    c = conn.cursor()
    try:
        c.execute("drop table if exists events;")
        c.execute("drop table if exists tickets ;")
        c.execute("drop table if exists users;")
        c.execute("drop table if exists mobile_address;")
        c.execute("drop table if exists exchange_txs;")

        c.execute('''create table events
                          (address text, owner text);''')

        c.execute('''create table tickets
                          (address text,
                          event_address text,
                          owner text,
                          create_hash text,
                          exchange_hash text);''')

        c.execute(
            '''create table users (address text, moniker text, passphrase
                            text);'''
        )

        c.execute(
            '''create table mobile_address (address text);'''
        )

        c.execute(
            '''create table exchange_txs (event_address text,
                                                tx_hash text);'''
        )

        conn.commit()
    except Error as e:
        logger.error(e)


def insert_exchange_tx(conn, event_address, tx_hash):
    conn.execute(
        ''' INSERT INTO exchange_txs (event_address, tx_hash)
                                VALUES(?,?);''',
        [event_address, tx_hash])
    conn.commit()


def to_dict(all_rows):
    res = []
    for row in all_rows:
        params = {item[0]: item[1] for item in list(zip(row.keys(), row))}
        res.append(params)
    return res


def insert_event(conn, event, owner):
    c = conn
    c.execute(
        ''' INSERT INTO Events(address, owner) VALUES(?,?);''',
        (event, owner),
    )
    conn.commit()


def insert_user(conn, address, moniker, passphrase):
    c = conn
    c.execute(
        ''' INSERT INTO Users(address, moniker, passphrase) VALUES(?,?,
                                    ?); ''',
        (address, moniker, passphrase),
    )
    conn.commit()


def if_moniker_exists(conn, moniker):
    c = conn.cursor()
    c.execute(
        '''SELECT * from Users where moniker=?''',
        [moniker],
    )
    if c.fetchone():
        return True
    else:
        return False


def select_address_by_moniker(conn, moniker):
    c = conn.cursor()
    c.execute(
        '''SELECT address from users where moniker=?''',
        [moniker],
    )
    res = c.fetchone()
    if res:
        return res[0]
    else:
        logger.error("User name doesn't exist for {}".format(moniker))


def select_all_events(conn):
    c = conn.cursor()
    c.execute('''select address from events;''')
    return [row[0] for row in c.fetchall()]


def select_exchange_txs(conn, event_address):
    c = conn.cursor()
    c.execute('''SELECT DISTINCT tx_hash from exchange_txs where event_address=?''',
              [event_address])
    return [row[0] for row in c.fetchall()]


def insert_mobile_address(conn, address):
    c = conn.cursor()
    c.execute("select * from mobile_address;")
    if c.fetchone():
        c.execute('''delete from mobile_address''')
    c.execute('''insert into mobile_address (address) values (?)''',
              [address], )
    conn.commit()


def delete_mobile_address(conn):
    conn.execute("delete from mobile_address")
    conn.commit()


def get_last_mobile_address(conn):
    c = conn.cursor()
    c.execute("select * from mobile_address;")
    res = c.fetchone()
    if res:
        return res[0]


if __name__ == '__main__':
    db = config.db_path
    logger.info('Connecting to db {}'.format(db))
    conn = sqlite3.connect(db)
    init_db(conn)
    conn.close()



from forge import protos
from forge.config import config

stub = protos.EventRpcStub(config.get_grpc_channel())


def subscribe(type=0, filter=None, req=None):
    """ RPC request to subscribe to specific type of transactions

    Note:
        This request might block the python thread

    Args:
        type(:obj:`TopicType`): type of topic to subscribe
        filter(string): filter for targetd transactions
        req(:obj:`RequestSubscribe`): completed request

    Returns:
        ResponseSubscribe

    """

    if req:
        res = stub.subscribe(req)

    else:
        res = stub.subscribe(
            protos.RequestSubscribe(type=type, filter=filter),
        )
    for r in res:
        yield r


def unsubscribe(topic, req=None):
    """GRPC call to stop previously subscribed transactions

    Args:
        topic(string): topic id returned in subscribe request
        req(:obj:`RequestUnsubscribe`): completed reques

    Returns:
        ResponseUnsubscribe

    """
    """

    Parameters
    ----------
    topic: str
    req: RequestUnsubscribe

    Returns
    -------
    ResponseUnsubscribe

    """
    if req:
        return stub.unsubscribe(req)
    else:
        return stub.unsubscribe(
            protos.RequestUnsubscribe(topic=topic),
        )
